# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["SubqueryCondition"]


class SubqueryCondition(BaseModel):
    """A subquery condition for EXISTS/NOT EXISTS filters"""

    from_: str = FieldInfo(alias="from")
    """Reference to the source view/table for the subquery"""

    where: List[str]
    """WHERE conditions for the subquery"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["RefWithLabel"]


class RefWithLabel(BaseModel):
    """A reference with optional label override"""

    ref: str
    """Reference using ref(), var(), or expr() syntax"""

    label: Optional[str] = None
    """Optional label override for this reference"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["InlineFieldParam"]


class InlineFieldParam(TypedDict, total=False):
    """An inline field definition for dimensions/measures/calculations"""

    kater_id: Required[str]
    """Unique identifier for this inline field"""

    name: Required[str]
    """Name of the inline field"""

    sql: Required[str]
    """SQL expression for the field"""

    label: Optional[str]
    """Human-readable label"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["CompilerResolveParams"]


class CompilerResolveParams(TypedDict, total=False):
    connection_id: Required[str]
    """Connection to resolve against"""

    query_ref: Required[str]
    """Reference to the query template (e.g. 'ref(MY_QUERY)')"""

    source: Optional[str]

    include_calculations: SequenceNotStr[str]
    """Calculation names to include"""

    include_dimensions: SequenceNotStr[str]
    """Dimension names to include"""

    include_filters: SequenceNotStr[str]
    """Filter names to include"""

    include_measures: SequenceNotStr[str]
    """Measure names to include"""

    variables: Dict[str, object]
    """User-selected variable values"""

    x_kater_cli_id: Annotated[str, PropertyInfo(alias="X-Kater-CLI-ID")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, Optional, cast

import httpx

from .groups import (
    GroupsResource,
    AsyncGroupsResource,
    GroupsResourceWithRawResponse,
    AsyncGroupsResourceWithRawResponse,
    GroupsResourceWithStreamingResponse,
    AsyncGroupsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, FileTypes, omit, not_given
from ...._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ...._compat import cached_property
from ....types.v1 import tenant_import_from_csv_params, tenant_import_from_warehouse_params
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.import_tenants_response import ImportTenantsResponse
from ....types.v1.tenant_get_tenants_schema_response import TenantGetTenantsSchemaResponse

__all__ = ["TenantsResource", "AsyncTenantsResource"]


class TenantsResource(SyncAPIResource):
    @cached_property
    def groups(self) -> GroupsResource:
        return GroupsResource(self._client)

    @cached_property
    def with_raw_response(self) -> TenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return TenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return TenantsResourceWithStreamingResponse(self)

    def get_tenants_schema(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantGetTenantsSchemaResponse:
        """
        Get all tenants as a TenantSchema object.

        Returns tenants in the YAML-compatible schema format with group references.
        Supports content negotiation: JSON by default, YAML with Accept:
        application/yaml.

        RLS: Filtered to current client (ClientRLSDB).
        """
        return self._get(
            "/api/v1/tenants/schema",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantGetTenantsSchemaResponse,
        )

    def import_from_csv(
        self,
        *,
        file: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImportTenantsResponse:
        """
        Import tenants from a CSV file using upsert semantics.

        Expected CSV format:

        - tenant_key (required): Unique tenant identifier
        - tenant_name (optional): Human-readable name
        - group_names (optional): Comma-separated list of group names

        Creates new tenants and updates existing ones with new name/groups. Groups are
        added additively (not replaced) and auto-created if needed.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: ValidationError: If CSV format is invalid (400)
        TenantCreationNotAllowedError: If tenancy type is NONE (400)

        Args:
          file: CSV file with tenant data

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal({"file": file})
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/api/v1/tenants/import/csv",
            body=maybe_transform(body, tenant_import_from_csv_params.TenantImportFromCsvParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ImportTenantsResponse,
        )

    def import_from_warehouse(
        self,
        *,
        connection_id: str,
        database: str,
        schema: str,
        table: str,
        tenant_key_column: str,
        tenant_group_column: Optional[str] | Omit = omit,
        tenant_name_column: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImportTenantsResponse:
        """
        Import tenants from a warehouse table using upsert semantics.

        Creates new tenants and updates existing ones with new name/groups. Groups are
        added additively (not replaced).

        Streams data from the warehouse in batches to handle large datasets without
        loading everything into memory.

        For tenants with groups, groups are aggregated per tenant key and auto-created
        if they don't exist.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: ConnectionNotFoundError: If the connection doesn't exist (404)
        TenantCreationNotAllowedError: If tenancy type is NONE (400)

        Args:
          connection_id: Warehouse connection ID to query

          database: Database name containing the tenant table

          schema: Schema name containing the tenant table

          table: Table name containing tenant data

          tenant_key_column: Column name for tenant key

          tenant_group_column: Column name for tenant group (optional)

          tenant_name_column: Column name for tenant display name (optional)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/tenants/import/warehouse",
            body=maybe_transform(
                {
                    "connection_id": connection_id,
                    "database": database,
                    "schema": schema,
                    "table": table,
                    "tenant_key_column": tenant_key_column,
                    "tenant_group_column": tenant_group_column,
                    "tenant_name_column": tenant_name_column,
                },
                tenant_import_from_warehouse_params.TenantImportFromWarehouseParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ImportTenantsResponse,
        )


class AsyncTenantsResource(AsyncAPIResource):
    @cached_property
    def groups(self) -> AsyncGroupsResource:
        return AsyncGroupsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncTenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncTenantsResourceWithStreamingResponse(self)

    async def get_tenants_schema(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantGetTenantsSchemaResponse:
        """
        Get all tenants as a TenantSchema object.

        Returns tenants in the YAML-compatible schema format with group references.
        Supports content negotiation: JSON by default, YAML with Accept:
        application/yaml.

        RLS: Filtered to current client (ClientRLSDB).
        """
        return await self._get(
            "/api/v1/tenants/schema",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantGetTenantsSchemaResponse,
        )

    async def import_from_csv(
        self,
        *,
        file: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImportTenantsResponse:
        """
        Import tenants from a CSV file using upsert semantics.

        Expected CSV format:

        - tenant_key (required): Unique tenant identifier
        - tenant_name (optional): Human-readable name
        - group_names (optional): Comma-separated list of group names

        Creates new tenants and updates existing ones with new name/groups. Groups are
        added additively (not replaced) and auto-created if needed.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: ValidationError: If CSV format is invalid (400)
        TenantCreationNotAllowedError: If tenancy type is NONE (400)

        Args:
          file: CSV file with tenant data

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal({"file": file})
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/api/v1/tenants/import/csv",
            body=await async_maybe_transform(body, tenant_import_from_csv_params.TenantImportFromCsvParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ImportTenantsResponse,
        )

    async def import_from_warehouse(
        self,
        *,
        connection_id: str,
        database: str,
        schema: str,
        table: str,
        tenant_key_column: str,
        tenant_group_column: Optional[str] | Omit = omit,
        tenant_name_column: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImportTenantsResponse:
        """
        Import tenants from a warehouse table using upsert semantics.

        Creates new tenants and updates existing ones with new name/groups. Groups are
        added additively (not replaced).

        Streams data from the warehouse in batches to handle large datasets without
        loading everything into memory.

        For tenants with groups, groups are aggregated per tenant key and auto-created
        if they don't exist.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: ConnectionNotFoundError: If the connection doesn't exist (404)
        TenantCreationNotAllowedError: If tenancy type is NONE (400)

        Args:
          connection_id: Warehouse connection ID to query

          database: Database name containing the tenant table

          schema: Schema name containing the tenant table

          table: Table name containing tenant data

          tenant_key_column: Column name for tenant key

          tenant_group_column: Column name for tenant group (optional)

          tenant_name_column: Column name for tenant display name (optional)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/tenants/import/warehouse",
            body=await async_maybe_transform(
                {
                    "connection_id": connection_id,
                    "database": database,
                    "schema": schema,
                    "table": table,
                    "tenant_key_column": tenant_key_column,
                    "tenant_group_column": tenant_group_column,
                    "tenant_name_column": tenant_name_column,
                },
                tenant_import_from_warehouse_params.TenantImportFromWarehouseParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ImportTenantsResponse,
        )


class TenantsResourceWithRawResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.get_tenants_schema = to_raw_response_wrapper(
            tenants.get_tenants_schema,
        )
        self.import_from_csv = to_raw_response_wrapper(
            tenants.import_from_csv,
        )
        self.import_from_warehouse = to_raw_response_wrapper(
            tenants.import_from_warehouse,
        )

    @cached_property
    def groups(self) -> GroupsResourceWithRawResponse:
        return GroupsResourceWithRawResponse(self._tenants.groups)


class AsyncTenantsResourceWithRawResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.get_tenants_schema = async_to_raw_response_wrapper(
            tenants.get_tenants_schema,
        )
        self.import_from_csv = async_to_raw_response_wrapper(
            tenants.import_from_csv,
        )
        self.import_from_warehouse = async_to_raw_response_wrapper(
            tenants.import_from_warehouse,
        )

    @cached_property
    def groups(self) -> AsyncGroupsResourceWithRawResponse:
        return AsyncGroupsResourceWithRawResponse(self._tenants.groups)


class TenantsResourceWithStreamingResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.get_tenants_schema = to_streamed_response_wrapper(
            tenants.get_tenants_schema,
        )
        self.import_from_csv = to_streamed_response_wrapper(
            tenants.import_from_csv,
        )
        self.import_from_warehouse = to_streamed_response_wrapper(
            tenants.import_from_warehouse,
        )

    @cached_property
    def groups(self) -> GroupsResourceWithStreamingResponse:
        return GroupsResourceWithStreamingResponse(self._tenants.groups)


class AsyncTenantsResourceWithStreamingResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.get_tenants_schema = async_to_streamed_response_wrapper(
            tenants.get_tenants_schema,
        )
        self.import_from_csv = async_to_streamed_response_wrapper(
            tenants.import_from_csv,
        )
        self.import_from_warehouse = async_to_streamed_response_wrapper(
            tenants.import_from_warehouse,
        )

    @cached_property
    def groups(self) -> AsyncGroupsResourceWithStreamingResponse:
        return AsyncGroupsResourceWithStreamingResponse(self._tenants.groups)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["ImportTenantsResponse", "Error"]


class Error(BaseModel):
    """Error for a single tenant during import."""

    code: str
    """Error code"""

    message: str
    """Error message"""

    tenant_key: str
    """Tenant key that failed"""


class ImportTenantsResponse(BaseModel):
    """Response model for tenant import operation."""

    total_found: int
    """Unique tenant keys found in source"""

    total_imported: int
    """New tenants created"""

    total_updated: int
    """Existing tenants updated"""

    errors: Optional[List[Error]] = None
    """Tenant-specific errors"""

    groups_created: Optional[List[str]] = None
    """Groups that were auto-created"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.tenants.group_get_tenant_groups_schema_response import GroupGetTenantGroupsSchemaResponse

__all__ = ["GroupsResource", "AsyncGroupsResource"]


class GroupsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> GroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return GroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return GroupsResourceWithStreamingResponse(self)

    def get_tenant_groups_schema(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GroupGetTenantGroupsSchemaResponse:
        """
        Get all tenant groups as a TenantGroupSchema object.

        Returns tenant groups in the YAML-compatible schema format. Supports content
        negotiation: JSON by default, YAML with Accept: application/yaml.

        RLS: Filtered to current client (ClientRLSDB).
        """
        return self._get(
            "/api/v1/tenants/groups/schema",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GroupGetTenantGroupsSchemaResponse,
        )


class AsyncGroupsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncGroupsResourceWithStreamingResponse(self)

    async def get_tenant_groups_schema(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GroupGetTenantGroupsSchemaResponse:
        """
        Get all tenant groups as a TenantGroupSchema object.

        Returns tenant groups in the YAML-compatible schema format. Supports content
        negotiation: JSON by default, YAML with Accept: application/yaml.

        RLS: Filtered to current client (ClientRLSDB).
        """
        return await self._get(
            "/api/v1/tenants/groups/schema",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GroupGetTenantGroupsSchemaResponse,
        )


class GroupsResourceWithRawResponse:
    def __init__(self, groups: GroupsResource) -> None:
        self._groups = groups

        self.get_tenant_groups_schema = to_raw_response_wrapper(
            groups.get_tenant_groups_schema,
        )


class AsyncGroupsResourceWithRawResponse:
    def __init__(self, groups: AsyncGroupsResource) -> None:
        self._groups = groups

        self.get_tenant_groups_schema = async_to_raw_response_wrapper(
            groups.get_tenant_groups_schema,
        )


class GroupsResourceWithStreamingResponse:
    def __init__(self, groups: GroupsResource) -> None:
        self._groups = groups

        self.get_tenant_groups_schema = to_streamed_response_wrapper(
            groups.get_tenant_groups_schema,
        )


class AsyncGroupsResourceWithStreamingResponse:
    def __init__(self, groups: AsyncGroupsResource) -> None:
        self._groups = groups

        self.get_tenant_groups_schema = async_to_streamed_response_wrapper(
            groups.get_tenant_groups_schema,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .manifest import Manifest
from ..._models import BaseModel
from .chart_config import ChartConfig
from .inline_field import InlineField
from .ref_with_label import RefWithLabel
from .subquery_condition import SubqueryCondition

__all__ = [
    "CompilerResolveResponse",
    "ResolvedQuery",
    "ResolvedQueryCalculation",
    "ResolvedQueryChartHint",
    "ResolvedQueryChartHintChartHint1",
    "ResolvedQueryChartHintChartHint2Output",
    "ResolvedQueryChartHintChartHint2OutputDefault",
    "ResolvedQueryDimension",
    "ResolvedQueryFilter",
    "ResolvedQueryFilterInlineFieldFilter",
    "ResolvedQueryFilterInlineExistsFilter1",
    "ResolvedQueryFilterInlineExistsFilter2",
    "ResolvedQueryMeasure",
    "ResolvedQueryOrderBy",
    "ResolvedQueryResolvedChart",
    "ResolvedQueryResolvedVariable",
    "ResolvedQueryResolvedVariableAllowedValues",
    "ResolvedQueryResolvedVariableAllowedValuesVariableAllowedValues1",
    "ResolvedQueryResolvedVariableAllowedValuesVariableAllowedValues1Static",
    "ResolvedQueryResolvedVariableAllowedValuesVariableAllowedValues2",
    "ResolvedQueryResolvedVariableConstraints",
    "DependencyGraph",
    "DependencyGraphNodes",
]

ResolvedQueryCalculation: TypeAlias = Union[RefWithLabel, InlineField, str]


class ResolvedQueryChartHintChartHint1(BaseModel):
    """A chart recommendation rule"""

    config: ChartConfig
    """Chart configuration with variable references"""

    recommend: Literal[
        "line", "bar", "stacked_bar", "area", "pie", "donut", "scatter", "table", "heatmap", "single_value"
    ]
    """Type of chart visualization"""

    when: Dict[str, Union[str, List[str]]]
    """
    Conditions based on variable values - can be single value (string) or multiple
    values (array)
    """


class ResolvedQueryChartHintChartHint2OutputDefault(BaseModel):
    config: ChartConfig
    """Chart configuration with variable references"""

    recommend: Literal[
        "line", "bar", "stacked_bar", "area", "pie", "donut", "scatter", "table", "heatmap", "single_value"
    ]
    """Type of chart visualization"""


class ResolvedQueryChartHintChartHint2Output(BaseModel):
    """A chart recommendation rule"""

    default: ResolvedQueryChartHintChartHint2OutputDefault


ResolvedQueryChartHint: TypeAlias = Union[ResolvedQueryChartHintChartHint1, ResolvedQueryChartHintChartHint2Output]

ResolvedQueryDimension: TypeAlias = Union[RefWithLabel, InlineField, str]


class ResolvedQueryFilterInlineFieldFilter(BaseModel):
    """An inline filter using field + operator + values"""

    field: str
    """Reference to the field to filter on"""

    name: str
    """Name of the inline filter"""

    operator: Literal[
        "equals",
        "not_equals",
        "in",
        "not_in",
        "greater_than",
        "less_than",
        "greater_than_or_equals",
        "less_than_or_equals",
        "between",
        "in_the_last",
        "in_the_next",
        "contains",
        "not_contains",
        "starts_with",
        "ends_with",
        "is_null",
        "is_not_null",
    ]
    """Filter operator to apply"""

    sql_value: Optional[str] = None
    """SQL expression for the filter value"""

    static_values: Optional[List[Union[str, float, bool]]] = None
    """Fixed values for the filter"""


class ResolvedQueryFilterInlineExistsFilter1(BaseModel):
    """An inline filter using EXISTS or NOT EXISTS with a subquery"""

    exists: SubqueryCondition
    """EXISTS subquery condition"""

    name: str
    """Name of the inline filter"""

    description: Optional[str] = None
    """Description of the filter"""

    label: Optional[str] = None
    """Human-readable label"""

    not_exists: Optional[SubqueryCondition] = None
    """A subquery condition for EXISTS/NOT EXISTS filters"""


class ResolvedQueryFilterInlineExistsFilter2(BaseModel):
    """An inline filter using EXISTS or NOT EXISTS with a subquery"""

    name: str
    """Name of the inline filter"""

    not_exists: SubqueryCondition
    """NOT EXISTS subquery condition"""

    description: Optional[str] = None
    """Description of the filter"""

    exists: Optional[SubqueryCondition] = None
    """A subquery condition for EXISTS/NOT EXISTS filters"""

    label: Optional[str] = None
    """Human-readable label"""


ResolvedQueryFilter: TypeAlias = Union[
    ResolvedQueryFilterInlineFieldFilter,
    str,
    ResolvedQueryFilterInlineExistsFilter1,
    ResolvedQueryFilterInlineExistsFilter2,
]

ResolvedQueryMeasure: TypeAlias = Union[RefWithLabel, InlineField, str]


class ResolvedQueryOrderBy(BaseModel):
    """Sort order specification for query results.

    Use desc for descending (highest/newest first) and asc for ascending (lowest/oldest first).
    """

    asc: Optional[List[str]] = None
    """Fields to sort in ascending order (lowest/oldest first)"""

    desc: Optional[List[str]] = None
    """Fields to sort in descending order (highest/newest first)"""


class ResolvedQueryResolvedChart(BaseModel):
    """The matched chart recommendation after evaluating chart hints"""

    config: ChartConfig
    """Chart configuration"""

    recommend: Literal[
        "line", "bar", "stacked_bar", "area", "pie", "donut", "scatter", "table", "heatmap", "single_value"
    ]
    """Recommended chart type"""


class ResolvedQueryResolvedVariableAllowedValuesVariableAllowedValues1Static(BaseModel):
    """A value with optional display label"""

    value: Union[str, float, bool]
    """The actual value"""

    label: Optional[str] = None
    """Human-readable label for the value"""


class ResolvedQueryResolvedVariableAllowedValuesVariableAllowedValues1(BaseModel):
    """Allowed values for a variable - either static list or from column"""

    static: List[ResolvedQueryResolvedVariableAllowedValuesVariableAllowedValues1Static]
    """Static list of allowed values with optional labels"""


class ResolvedQueryResolvedVariableAllowedValuesVariableAllowedValues2(BaseModel):
    """Allowed values for a variable - either static list or from column"""

    from_column: str
    """Reference to column for dynamic values"""

    cache_ttl: Optional[int] = None
    """Cache time-to-live in seconds"""

    limit: Optional[int] = None
    """Maximum number of values to retrieve"""

    order_by: Optional[Literal["asc", "desc"]] = None
    """Sort order for values"""


ResolvedQueryResolvedVariableAllowedValues: TypeAlias = Union[
    ResolvedQueryResolvedVariableAllowedValuesVariableAllowedValues1,
    ResolvedQueryResolvedVariableAllowedValuesVariableAllowedValues2,
    None,
]


class ResolvedQueryResolvedVariableConstraints(BaseModel):
    """Constraints for variable types"""

    max: Optional[float] = None
    """Maximum allowed value"""

    max_length: Optional[int] = None
    """Maximum length for STRING variables"""

    min: Optional[float] = None
    """Minimum allowed value"""

    step: Optional[float] = None
    """Step increment for numeric input"""


class ResolvedQueryResolvedVariable(BaseModel):
    """A variable definition with its bound value"""

    bound_value: Union[str, float, bool]
    """The concrete value bound for this resolution"""

    default: Union[str, float, bool]
    """Default value for this variable"""

    name: str
    """Variable name identifier"""

    type: Literal[
        "STRING", "INT", "FLOAT", "DATE", "TIMESTAMP", "BOOL", "DIMENSION", "MEASURE", "CALCULATION", "FILTER"
    ]
    """Data type of the variable"""

    allowed_values: Optional[ResolvedQueryResolvedVariableAllowedValues] = None
    """Allowed values configuration"""

    constraints: Optional[ResolvedQueryResolvedVariableConstraints] = None
    """Constraints for variable types"""

    description: Optional[str] = None
    """Description of the variable's purpose"""

    is_default: Optional[bool] = None
    """True if bound_value equals the default value"""

    label: Optional[str] = None
    """Human-readable label for the variable"""


class ResolvedQuery(BaseModel):
    """The fully resolved query object"""

    kater_id: str
    """Unique identifier for this resolved query instance"""

    name: str
    """Name from the leaf query in the inheritance chain"""

    source_query: str
    """Reference to the original query template this was resolved from"""

    topic: str
    """
    Reference to the topic this query uses (always known after inheritance
    resolution)
    """

    ai_context: Optional[str] = None
    """Usage guidance for AI processing"""

    calculations: Optional[List[ResolvedQueryCalculation]] = None
    """Merged required + selected optional calculations"""

    chart_hints: Optional[List[ResolvedQueryChartHint]] = None
    """Chart recommendations preserved for evaluation"""

    custom_properties: Optional[Dict[str, object]] = None
    """Custom properties"""

    description: Optional[str] = None
    """Description of the query"""

    dimensions: Optional[List[ResolvedQueryDimension]] = None
    """Merged required + selected optional dimensions"""

    filters: Optional[List[ResolvedQueryFilter]] = None
    """Merged required + selected optional filters"""

    inheritance_chain: Optional[List[str]] = None
    """Ordered list of query refs that were merged during inheritance resolution"""

    label: Optional[str] = None
    """Human-readable label with var() values substituted"""

    limit: Optional[int] = None
    """Maximum number of rows to return"""

    measures: Optional[List[ResolvedQueryMeasure]] = None
    """Merged required + selected optional measures"""

    order_by: Optional[ResolvedQueryOrderBy] = None
    """Sort order specification for query results.

    Use desc for descending (highest/newest first) and asc for ascending
    (lowest/oldest first).
    """

    required_access_grants: Optional[List[str]] = None
    """Access grants required to use this query"""

    resolved_chart: Optional[ResolvedQueryResolvedChart] = None
    """The matched chart recommendation after evaluating chart hints"""

    resolved_variables: Optional[List[ResolvedQueryResolvedVariable]] = None
    """Full variable definitions with bound values"""

    widget_category: Optional[Literal["axis", "pie", "single_value", "heatmap", "table", "static"]] = None
    """Category of widget that determines data shape constraints for queries"""


class DependencyGraphNodes(BaseModel):
    """A node in the dependency graph."""

    file: str
    """Source file path"""

    fqn: str
    """Fully qualified name (e.g. 'dim_customer.region')"""

    kater_id: str
    """UUID of the schema object"""

    line: int
    """Line number in source file"""

    node_type: str
    """Node type: QUERY, VIEW, DIMENSION, MEASURE, FILTER, EXPRESSION"""


class DependencyGraph(BaseModel):
    """Dependency graph between schema objects."""

    edges: Dict[str, Dict[str, List[str]]]
    """Edge relationships with UUID string keys"""

    nodes: Dict[str, DependencyGraphNodes]
    """UUID string to node mapping"""


class CompilerResolveResponse(BaseModel):
    """Response model for a resolved query."""

    resolved_query: ResolvedQuery
    """The fully resolved query object"""

    dependency_graph: Optional[DependencyGraph] = None
    """Dependency graph between schema objects."""

    manifest: Optional[Manifest] = None
    """Compilation manifest with all named objects."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .manifest import Manifest
from ..._models import BaseModel
from .compiler_error_item import CompilerErrorItem

__all__ = ["CompilerCompileResponse", "Metadata"]


class Metadata(BaseModel):
    """Compilation metadata from the compiler."""

    dialect: str
    """SQL dialect used (e.g. 'snowflake')"""

    query_ref: str
    """Reference to the compiled query"""

    dimensions_used: Optional[List[str]] = None
    """Dimension names used in compilation"""

    filters_used: Optional[List[str]] = None
    """Filter names used in compilation"""

    measures_used: Optional[List[str]] = None
    """Measure names used in compilation"""

    views_used: Optional[List[str]] = None
    """View names used in compilation"""


class CompilerCompileResponse(BaseModel):
    """Response model for SQL compilation."""

    dialect: str
    """SQL dialect used (e.g. 'snowflake')"""

    success: bool
    """Whether compilation succeeded"""

    errors: Optional[List[CompilerErrorItem]] = None
    """Compilation errors"""

    manifest: Optional[Manifest] = None
    """Compilation manifest with all named objects."""

    metadata: Optional[Metadata] = None
    """Compilation metadata from the compiler."""

    sql: Optional[str] = None
    """Generated SQL statement"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, strip_not_given, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import compiler_compile_params, compiler_resolve_params, compiler_validate_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.v1.compiler_compile_response import CompilerCompileResponse
from ...types.v1.compiler_resolve_response import CompilerResolveResponse
from ...types.v1.compiler_validate_response import CompilerValidateResponse

__all__ = ["CompilerResource", "AsyncCompilerResource"]


class CompilerResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CompilerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return CompilerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CompilerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return CompilerResourceWithStreamingResponse(self)

    def compile(
        self,
        *,
        connection_id: str,
        resolved_query: compiler_compile_params.ResolvedQuery,
        source: Optional[str] | Omit = omit,
        tenant_database: Optional[str] | Omit = omit,
        x_kater_cli_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompilerCompileResponse:
        """
        Compile a resolved query to SQL.

        Takes a previously resolved query and generates the final SQL statement for the
        target dialect.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          connection_id: Connection to compile against

          resolved_query: Previously resolved query object from /resolve

          tenant_database: Optional tenant database override

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"X-Kater-CLI-ID": x_kater_cli_id}), **(extra_headers or {})}
        return self._post(
            "/api/v1/compiler/compile",
            body=maybe_transform(
                {
                    "connection_id": connection_id,
                    "resolved_query": resolved_query,
                    "tenant_database": tenant_database,
                },
                compiler_compile_params.CompilerCompileParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"source": source}, compiler_compile_params.CompilerCompileParams),
            ),
            cast_to=CompilerCompileResponse,
        )

    def resolve(
        self,
        *,
        connection_id: str,
        query_ref: str,
        source: Optional[str] | Omit = omit,
        include_calculations: SequenceNotStr[str] | Omit = omit,
        include_dimensions: SequenceNotStr[str] | Omit = omit,
        include_filters: SequenceNotStr[str] | Omit = omit,
        include_measures: SequenceNotStr[str] | Omit = omit,
        variables: Dict[str, object] | Omit = omit,
        x_kater_cli_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompilerResolveResponse:
        """
        Resolve a query template with user-selected parameters.

        Takes a query reference and variable selections, returns the fully resolved
        query object ready for compilation.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          connection_id: Connection to resolve against

          query_ref: Reference to the query template (e.g. 'ref(MY_QUERY)')

          include_calculations: Calculation names to include

          include_dimensions: Dimension names to include

          include_filters: Filter names to include

          include_measures: Measure names to include

          variables: User-selected variable values

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"X-Kater-CLI-ID": x_kater_cli_id}), **(extra_headers or {})}
        return self._post(
            "/api/v1/compiler/resolve",
            body=maybe_transform(
                {
                    "connection_id": connection_id,
                    "query_ref": query_ref,
                    "include_calculations": include_calculations,
                    "include_dimensions": include_dimensions,
                    "include_filters": include_filters,
                    "include_measures": include_measures,
                    "variables": variables,
                },
                compiler_resolve_params.CompilerResolveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"source": source}, compiler_resolve_params.CompilerResolveParams),
            ),
            cast_to=CompilerResolveResponse,
        )

    def validate(
        self,
        *,
        source: Optional[str] | Omit = omit,
        connection_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        x_kater_cli_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompilerValidateResponse:
        """
        Validate a schema file set against a connection.

        Checks all views, queries, and related schemas for correctness and returns any
        errors or warnings found.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          connection_ids: Optional connection IDs to validate. If omitted, validates all connections.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"X-Kater-CLI-ID": x_kater_cli_id}), **(extra_headers or {})}
        return self._post(
            "/api/v1/compiler/validate",
            body=maybe_transform({"connection_ids": connection_ids}, compiler_validate_params.CompilerValidateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"source": source}, compiler_validate_params.CompilerValidateParams),
            ),
            cast_to=CompilerValidateResponse,
        )


class AsyncCompilerResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCompilerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncCompilerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCompilerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncCompilerResourceWithStreamingResponse(self)

    async def compile(
        self,
        *,
        connection_id: str,
        resolved_query: compiler_compile_params.ResolvedQuery,
        source: Optional[str] | Omit = omit,
        tenant_database: Optional[str] | Omit = omit,
        x_kater_cli_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompilerCompileResponse:
        """
        Compile a resolved query to SQL.

        Takes a previously resolved query and generates the final SQL statement for the
        target dialect.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          connection_id: Connection to compile against

          resolved_query: Previously resolved query object from /resolve

          tenant_database: Optional tenant database override

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"X-Kater-CLI-ID": x_kater_cli_id}), **(extra_headers or {})}
        return await self._post(
            "/api/v1/compiler/compile",
            body=await async_maybe_transform(
                {
                    "connection_id": connection_id,
                    "resolved_query": resolved_query,
                    "tenant_database": tenant_database,
                },
                compiler_compile_params.CompilerCompileParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"source": source}, compiler_compile_params.CompilerCompileParams),
            ),
            cast_to=CompilerCompileResponse,
        )

    async def resolve(
        self,
        *,
        connection_id: str,
        query_ref: str,
        source: Optional[str] | Omit = omit,
        include_calculations: SequenceNotStr[str] | Omit = omit,
        include_dimensions: SequenceNotStr[str] | Omit = omit,
        include_filters: SequenceNotStr[str] | Omit = omit,
        include_measures: SequenceNotStr[str] | Omit = omit,
        variables: Dict[str, object] | Omit = omit,
        x_kater_cli_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompilerResolveResponse:
        """
        Resolve a query template with user-selected parameters.

        Takes a query reference and variable selections, returns the fully resolved
        query object ready for compilation.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          connection_id: Connection to resolve against

          query_ref: Reference to the query template (e.g. 'ref(MY_QUERY)')

          include_calculations: Calculation names to include

          include_dimensions: Dimension names to include

          include_filters: Filter names to include

          include_measures: Measure names to include

          variables: User-selected variable values

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"X-Kater-CLI-ID": x_kater_cli_id}), **(extra_headers or {})}
        return await self._post(
            "/api/v1/compiler/resolve",
            body=await async_maybe_transform(
                {
                    "connection_id": connection_id,
                    "query_ref": query_ref,
                    "include_calculations": include_calculations,
                    "include_dimensions": include_dimensions,
                    "include_filters": include_filters,
                    "include_measures": include_measures,
                    "variables": variables,
                },
                compiler_resolve_params.CompilerResolveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"source": source}, compiler_resolve_params.CompilerResolveParams),
            ),
            cast_to=CompilerResolveResponse,
        )

    async def validate(
        self,
        *,
        source: Optional[str] | Omit = omit,
        connection_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        x_kater_cli_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CompilerValidateResponse:
        """
        Validate a schema file set against a connection.

        Checks all views, queries, and related schemas for correctness and returns any
        errors or warnings found.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          connection_ids: Optional connection IDs to validate. If omitted, validates all connections.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"X-Kater-CLI-ID": x_kater_cli_id}), **(extra_headers or {})}
        return await self._post(
            "/api/v1/compiler/validate",
            body=await async_maybe_transform(
                {"connection_ids": connection_ids}, compiler_validate_params.CompilerValidateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"source": source}, compiler_validate_params.CompilerValidateParams),
            ),
            cast_to=CompilerValidateResponse,
        )


class CompilerResourceWithRawResponse:
    def __init__(self, compiler: CompilerResource) -> None:
        self._compiler = compiler

        self.compile = to_raw_response_wrapper(
            compiler.compile,
        )
        self.resolve = to_raw_response_wrapper(
            compiler.resolve,
        )
        self.validate = to_raw_response_wrapper(
            compiler.validate,
        )


class AsyncCompilerResourceWithRawResponse:
    def __init__(self, compiler: AsyncCompilerResource) -> None:
        self._compiler = compiler

        self.compile = async_to_raw_response_wrapper(
            compiler.compile,
        )
        self.resolve = async_to_raw_response_wrapper(
            compiler.resolve,
        )
        self.validate = async_to_raw_response_wrapper(
            compiler.validate,
        )


class CompilerResourceWithStreamingResponse:
    def __init__(self, compiler: CompilerResource) -> None:
        self._compiler = compiler

        self.compile = to_streamed_response_wrapper(
            compiler.compile,
        )
        self.resolve = to_streamed_response_wrapper(
            compiler.resolve,
        )
        self.validate = to_streamed_response_wrapper(
            compiler.validate,
        )


class AsyncCompilerResourceWithStreamingResponse:
    def __init__(self, compiler: AsyncCompilerResource) -> None:
        self._compiler = compiler

        self.compile = async_to_streamed_response_wrapper(
            compiler.compile,
        )
        self.resolve = async_to_streamed_response_wrapper(
            compiler.resolve,
        )
        self.validate = async_to_streamed_response_wrapper(
            compiler.validate,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["TenantImportFromWarehouseParams"]


class TenantImportFromWarehouseParams(TypedDict, total=False):
    connection_id: Required[str]
    """Warehouse connection ID to query"""

    database: Required[str]
    """Database name containing the tenant table"""

    schema: Required[str]
    """Schema name containing the tenant table"""

    table: Required[str]
    """Table name containing tenant data"""

    tenant_key_column: Required[str]
    """Column name for tenant key"""

    tenant_group_column: Optional[str]
    """Column name for tenant group (optional)"""

    tenant_name_column: Optional[str]
    """Column name for tenant display name (optional)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["RefWithLabelParam"]


class RefWithLabelParam(TypedDict, total=False):
    """A reference with optional label override"""

    ref: Required[str]
    """Reference using ref(), var(), or expr() syntax"""

    label: Optional[str]
    """Optional label override for this reference"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from kater import Kater, AsyncKater
from tests.utils import assert_matches_type
from kater.types.v1 import (
    ImportTenantsResponse,
    TenantGetTenantsSchemaResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTenants:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_tenants_schema(self, client: Kater) -> None:
        tenant = client.v1.tenants.get_tenants_schema()
        assert_matches_type(TenantGetTenantsSchemaResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_tenants_schema(self, client: Kater) -> None:
        response = client.v1.tenants.with_raw_response.get_tenants_schema()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = response.parse()
        assert_matches_type(TenantGetTenantsSchemaResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_tenants_schema(self, client: Kater) -> None:
        with client.v1.tenants.with_streaming_response.get_tenants_schema() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = response.parse()
            assert_matches_type(TenantGetTenantsSchemaResponse, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_import_from_csv(self, client: Kater) -> None:
        tenant = client.v1.tenants.import_from_csv(
            file=b"raw file contents",
        )
        assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_import_from_csv(self, client: Kater) -> None:
        response = client.v1.tenants.with_raw_response.import_from_csv(
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = response.parse()
        assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_import_from_csv(self, client: Kater) -> None:
        with client.v1.tenants.with_streaming_response.import_from_csv(
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = response.parse()
            assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_import_from_warehouse(self, client: Kater) -> None:
        tenant = client.v1.tenants.import_from_warehouse(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database="x",
            schema="x",
            table="x",
            tenant_key_column="x",
        )
        assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_import_from_warehouse_with_all_params(self, client: Kater) -> None:
        tenant = client.v1.tenants.import_from_warehouse(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database="x",
            schema="x",
            table="x",
            tenant_key_column="x",
            tenant_group_column="tenant_group_column",
            tenant_name_column="tenant_name_column",
        )
        assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_import_from_warehouse(self, client: Kater) -> None:
        response = client.v1.tenants.with_raw_response.import_from_warehouse(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database="x",
            schema="x",
            table="x",
            tenant_key_column="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = response.parse()
        assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_import_from_warehouse(self, client: Kater) -> None:
        with client.v1.tenants.with_streaming_response.import_from_warehouse(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database="x",
            schema="x",
            table="x",
            tenant_key_column="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = response.parse()
            assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTenants:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_tenants_schema(self, async_client: AsyncKater) -> None:
        tenant = await async_client.v1.tenants.get_tenants_schema()
        assert_matches_type(TenantGetTenantsSchemaResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_tenants_schema(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.tenants.with_raw_response.get_tenants_schema()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = await response.parse()
        assert_matches_type(TenantGetTenantsSchemaResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_tenants_schema(self, async_client: AsyncKater) -> None:
        async with async_client.v1.tenants.with_streaming_response.get_tenants_schema() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = await response.parse()
            assert_matches_type(TenantGetTenantsSchemaResponse, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_import_from_csv(self, async_client: AsyncKater) -> None:
        tenant = await async_client.v1.tenants.import_from_csv(
            file=b"raw file contents",
        )
        assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_import_from_csv(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.tenants.with_raw_response.import_from_csv(
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = await response.parse()
        assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_import_from_csv(self, async_client: AsyncKater) -> None:
        async with async_client.v1.tenants.with_streaming_response.import_from_csv(
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = await response.parse()
            assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_import_from_warehouse(self, async_client: AsyncKater) -> None:
        tenant = await async_client.v1.tenants.import_from_warehouse(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database="x",
            schema="x",
            table="x",
            tenant_key_column="x",
        )
        assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_import_from_warehouse_with_all_params(self, async_client: AsyncKater) -> None:
        tenant = await async_client.v1.tenants.import_from_warehouse(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database="x",
            schema="x",
            table="x",
            tenant_key_column="x",
            tenant_group_column="tenant_group_column",
            tenant_name_column="tenant_name_column",
        )
        assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_import_from_warehouse(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.tenants.with_raw_response.import_from_warehouse(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database="x",
            schema="x",
            table="x",
            tenant_key_column="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = await response.parse()
        assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_import_from_warehouse(self, async_client: AsyncKater) -> None:
        async with async_client.v1.tenants.with_streaming_response.import_from_warehouse(
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database="x",
            schema="x",
            table="x",
            tenant_key_column="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = await response.parse()
            assert_matches_type(ImportTenantsResponse, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["CompilerErrorItem"]


class CompilerErrorItem(BaseModel):
    """A single compiler validation or compilation error."""

    code: str
    """Machine-readable error code"""

    message: str
    """Human-readable error description"""

    file: Optional[str] = None
    """Source file path where the error occurred"""

    line: Optional[int] = None
    """Line number in the source file"""

    ref: Optional[str] = None
    """Reference to the source element (e.g. view or query name)"""

    remediation: Optional[str] = None
    """Suggested fix for this error"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .connection import Connection

__all__ = ["ConnectionListConnectionsResponse"]

ConnectionListConnectionsResponse: TypeAlias = List[Connection]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ManifestEntry"]


class ManifestEntry(BaseModel):
    """A single object entry in the manifest."""

    kater_id: str

    name: str

    type: str

    label: Optional[str] = None

    parent_id: Optional[str] = None

    source_file: Optional[str] = None
